
html.default <- NULL

html.file <- function(file="report.html") { 
	newhtml <- list(file=file)
	oldhtml <- html.default
	env <- .GlobalEnv
	assign("html.default",newhtml,env)
	.GlobalEnv <- env

	invisible(oldhtml)
}


html.write <- function(text,file=html.default$file,append=TRUE) {
	cat(text,file=file,append=append)
} 


# generic function for beginning a tag
tag.begin <- function(name, file=html.default$file, ...) {
	# start the tag
	tag <- paste(c("<",name), collapse="")
	
	# build the attributes
	attr.list <- list(...)
	for(n in names(attr.list)) {
		attr <- paste(c(n,"=",'"',attr.list[n],'"'),collapse="")
		# add to the end of tag
		tag <- paste(c(tag,attr),collapse=" ")
	}
	
	# end the tag
	tag <- paste(c(tag, ">"), collapse="")
	html.write(tag,file)
}


# generic function for ending a tag
tag.end <- function(name,file=html.default$file) {
	tag <- paste(c("</",name,">"), collapse="")
	html.write(tag,file)
}




# begins an html document
html.begin <- function(title, file=html.default$file, bgcolor="#FFFFFF") {
	html.write("<!-- HTML output generated by Zed A. Shaw's HTML report library. -->",file, append=FALSE)
	tag.begin("html",file)
	tag.begin("head",file)
	tag.begin("title",file)
	html.write(title, file)
	tag.end("title",file)
	tag.end("head",file)
	tag.begin("body",file,bgcolor=bgcolor)
}


# begins an html header (H) tag
html.h <- function(title, file=html.default$file, level=1) {
	h <- paste(c("<h",level,">",title,"</h",level,">"),collapse="")
	html.write(h,file)
}


html.p <- function(text="",file=html.default$file) {
	p <- paste(c("<p>",text,"</p>"),collapse="")
	html.write(p,file)
}


html.list <- function(list.elements,file=html.default$file,type="ul") {
	tag.begin(type,file)
	
	for(item in list.elements) {
          tag.begin("li",file)
          html.write(item,file)
          tag.end("li",file)
        }
	
	tag.end(type,file)
}

# writes an HR tag
html.hr <- function(file=html.default$file) {
	tag.begin("hr",file)
	tag.end("hr",file)
}


# writes an img tag
html.img <- function(src,file=html.default$file,border="0") {
	tag.begin("img",file,src=src,border=border)
	tag.end("img",file)
}

# ends an html document
html.end <- function(file=html.default$file) {
	tag.end("body",file)
	tag.end("html",file)
}


# sends the output to the html report file
html.divert <- function(file=html.default$file) {
  sink(file,append=TRUE)
}

# stops sending the output to the report file
html.revert <- function() {
  sink()
}
