.First.lib <- function(lib, pkg) {
  require("xtable")
}


report.default <- NULL

##
# Takes new parameters and sets the default target report to them.  It returns the 
# the report object that was previously the default.
##
report.new <- function(reporttitle,file="report.html",type="html") {
 
	rep <-  list(file=paste(c(getwd(),"/",file),collapse=""),title=reporttitle)
	class(rep) <- "report"


	# save the old one so we can return it
	oldrep <- report.default

	# get the global environment and set our new report.default
	env <- .GlobalEnv
	assign("report.default",rep,env)
	.GlobalEnv <- env

	invisible(oldrep)
}




##
# Sets the default report to another report object, and returns the previous
# default report object.
##
report.set <- function(newreport) {
	oldrep <- report.default
	report.default <- newreport
	return(oldrep)
}


##
# Internal function that starts the report defined in the report object.
##
report.begin <- function(report=report.default) {
  # now we begin the report proper
  html.file(report$file)
  html.begin(report$title)
  html.h(report$title)
}






##
# Internal function that ends the report defined in the report object.
##
report.end <- function(report=report.default) {
  html.end(report$file)
}






##
# this opens the netscape window on the file, assuming it is named properly
# This was blatantly stolen from Robert Gentleman <rgentlem@jimmy.harvard.edu>
##
report.view <- function(report=report.default) {
    query <- report$file

    OST <- .Platform$OS.type
    if (OST == "windows") {
        shell.exec(query)
    }
    else if (OST == "unix") {
        if (is.null(getOption("browser"))) {
	  stop("options(\"browser\") not set")
	}

        browser <- getOption("browser")
        system(paste(browser, " -remote \"openURL(", query, ")\"2>/dev/null || ", browser, " ", query, " &", sep = ""))
    }
    else {
        msg <- paste("don't know how to open the browser on", 
            OST)
        stop(msg)
    }
    invisible(NULL)
}



##
# Generic dispatch method that writes various report sections
# based on the type of object given in x.
##
report <- function(x,...) {
  UseMethod("report")
}




################### generic functions ##########################


##
# Writes tables to the report
##
report.xtable <- function(x,descr="",heading="Table", ...) {
  # write out the report
  html.h(heading,report.default$file)
  html.p(descr,report.default$file)
  print(x,type="html",report.default$file,append=TRUE)
}






report.data.frame <- function(x,desc="",heading="Data Frame", ...) {
  report.xtable(xtable(x),report,desc,heading,...)
}


report.summary.lm <- function(x,desc="",heading="LM Detailed Summary", ...) {
  html.h(heading ,report.default$file)
  html.p(desc,report.default$file)
  tag.begin("pre",report.default$file)
  html.divert(report.default$file)
  print(x)
  html.revert()
  tag.end("pre",report.default$file)
}






# For some reason, xtable can't do tables...
report.table <- function(x,desc="",heading="Generic Table") {
  html.h(heading,report.default$file)
  html.p(desc,report.default$file)
  tag.begin("pre",report.default$file)
  html.divert(report.default$file)
  print(x)
  html.revert()
  tag.end("pre",report.default$file)
}

report.lm <- function(x,desc="",heading="Linear Model",...) {
  report.xtable(xtable(x),desc,heading)
}

report.matrix <- function(x,desc="",heading="Matrix",...) {
    report.xtable(xtable(x),desc,heading)
}

report.anova <- function(x,desc="",heading="ANOVA",...) {
    report.xtable(xtable(x),desc,heading)
}

report.aov <- function(x,desc="",heading="AOV",...) {
    report.xtable(xtable(x),desc,heading)
}

report.summary.aov <- function(x,desc="",heading="AOV Detailed Summary",...) {
    report.xtable(xtable(x),desc,heading)
}

report.aovlist <- function(x,desc="",heading="AOV List",...) {
    report.xtable(xtable(x),desc,heading)
}

report.summary.aovlist <- function(x,desc="",heading="AOV List Detailed Summary",...) {
    report.xtable(xtable(x),desc,heading)
}


report.glm <- function(x,desc="",heading="Generalized Linear Model",...) {
    report.xtable(xtable(x),desc,heading)
}

report.summary.glm <- function(x,desc="",heading="GLM Detailed Summary",...) {
    report.xtable(xtable(x),desc,heading)
}

report.prcomp <- function(x,desc="",heading="PRComp",...) {
    report.xtable(xtable(x),desc,heading)
}

report.summary.prcomp <- function(x,desc="",heading="PRComp Detailed Summary",...) {
    report.xtable(xtable(x),desc,heading)
}


# special reporting function that will write out the given plot
# probably only useful for very simplistic plot situations
report.plot <- function(file,data,plotfunc=plot,desc="",heading="Figure",...) {
  png(filename = file,...)
  plotfunc(data)
  dev.off()

  # write the image to the report
  html.h(heading, report.default$file)
  html.p(desc,report.default$file)
  html.img(file,report.default$file) 
}
